@@ -36,6 +36,7 @@
 #include "kernel-shared/uapi/btrfs.h"
 #include "kernel-shared/ctree.h"
 #include "common/open-utils.h"
+#include "common/sysfs-utils.h"
 #include "common/utils.h"
 #include "common/help.h"
 #include "common/units.h"
@@ -834,7 +835,7 @@ static struct btrfs_qgroup *get_or_add_qgroup(int fd,
 		enum btrfs_util_error uret;
 		char *path;
 
-		uret = btrfs_util_subvolume_path_fd(fd, qgroupid, &path);
+		uret = btrfs_util_subvolume_get_path_fd(fd, qgroupid, &path);
 		if (uret == BTRFS_UTIL_OK)
 			bq->path = path;
 		else if (uret != BTRFS_UTIL_ERROR_SUBVOLUME_NOT_FOUND) {
@@ -851,7 +852,7 @@ static struct btrfs_qgroup *get_or_add_qgroup(int fd,
 		 * Passing @subvol as NULL will force the search to only search
 		 * for the ROOT_ITEM.
 		 */
-		uret = btrfs_util_subvolume_info_fd(fd, qgroupid, NULL);
+		uret = btrfs_util_subvolume_get_info_fd(fd, qgroupid, NULL);
 		if (uret == BTRFS_UTIL_OK) {
 			bq->stale = false;
 		} else if (uret == BTRFS_UTIL_ERROR_SUBVOLUME_NOT_FOUND) {
@@ -1610,6 +1611,31 @@ static void print_all_qgroups_json(struct qgroup_lookup *qgroup_lookup)
 	fmt_end(&fctx);
 }
 
+/*
+ * Tree search based 'inconsistent' flag is only updated at transaction commit
+ * time.  Thus even if the qgroup_status flag shows consistent, the qgroup may
+ * already be in an inconsistent state.
+ */
+static void check_qgroup_sysfs_inconsistent(int fd, const struct qgroup_lookup *qgroup_lookup)
+{
+	u64 value;
+	int sysfs_fd;
+	int ret;
+
+	if (qgroup_lookup->flags & BTRFS_QGROUP_STATUS_FLAG_INCONSISTENT)
+		return;
+	sysfs_fd = sysfs_open_fsid_file(fd, "qgroups/inconsistent");
+	if (fd < 0)
+		return;
+	ret = sysfs_read_fsid_file_u64(fd, "qgroups/inconsistent", &value);
+	if (ret < 0)
+		goto out;
+	if (value)
+		warning("qgroup data inconsistent, rescan recommended");
+out:
+	close(sysfs_fd);
+}
+
 static int show_qgroups(int fd,
 		       struct btrfs_qgroup_filter_set *filter_set,
 		       struct btrfs_qgroup_comparer_set *comp_set)
@@ -1622,6 +1648,8 @@ static int show_qgroups(int fd,
 	ret = qgroups_search_all(fd, &qgroup_lookup);
 	if (ret)
 		return ret;
+
+	check_qgroup_sysfs_inconsistent(fd, &qgroup_lookup);
 	__filter_and_sort_qgroups(&qgroup_lookup, &sort_tree,
 				  filter_set, comp_set);
 	if (bconf.output_format == CMD_FORMAT_JSON)
@@ -2016,7 +2044,7 @@ static int cmd_qgroup_show(const struct cmd_struct *cmd, int argc, char **argv)
 	}
 
 	if (sync) {
-		err = btrfs_util_sync_fd(fd);
+		err = btrfs_util_fs_sync_fd(fd);
 		if (err)
 			warning("sync ioctl failed on '%s': %m", path);
 	}
@@ -2108,7 +2136,7 @@ static int cmd_qgroup_limit(const struct cmd_struct *cmd, int argc, char **argv)
 	if (argc - optind == 2) {
 		args.qgroupid = 0;
 		path = argv[optind + 1];
-		err = btrfs_util_is_subvolume(path);
+		err = btrfs_util_subvolume_is_valid(path);
 		if (err) {
 			error_btrfs_util(err);
 			return 1;
@@ -2242,7 +2270,7 @@ static int cmd_qgroup_clear_stale(const struct cmd_struct *cmd, int argc, char *
 	}
 
 	/* Sync the fs so that the qgroup numbers are uptodate. */
-	err = btrfs_util_sync_fd(fd);
+	err = btrfs_util_fs_sync_fd(fd);
 	if (err)
 		warning("syncing filesystem failed: %m");
 
